import java.io.*;
import java.util.*;
import java.util.concurrent.TimeUnit;

public class Pearson implements Runnable {
	public StringTokenizer strtok;
	public BufferedReader inr;
	public PrintWriter out;

	public static void main(String[] args) {
		new Thread(new Pearson()).start();
	}

	public static final String taskname = "pearson";

	public void run() {
		try {
			inr = new BufferedReader(new FileReader(taskname + ".in"));
			out = new PrintWriter(new FileWriter(taskname + ".out"));
			solve();
		} catch (Exception e) {
			e.printStackTrace();
			System.exit(9000);
		} finally {
			out.flush();
			out.close();
		}
	}

	public String nextToken() throws IOException {
		while (strtok == null || !strtok.hasMoreTokens()) {
			strtok = new StringTokenizer(inr.readLine());
		}
		return strtok.nextToken();
	}

	// Solution
	
	Token[] kw;
	int n;
	
	public void readTokens() {
		ArrayList<Token> arr = new ArrayList<Token>();
		try {
			int i = 0;
			while(true)
				arr.add(new Token(nextToken(), i++));
		} catch (Exception e) {}
		kw = new Token[arr.size()];
		for (int i = 0; i < arr.size(); i++) {
			kw[i] = arr.get(i);
		}
		Arrays.sort(kw);
		n = arr.size();
	}
	
	int[] T;
	boolean[] used;

	public boolean swap(int i, int j) {
		if (used[i] || used[j])
			return false;
		int t = T[i];
		T[i] = T[j];
		T[j] = t;
		return true;
	}
	
	int hash;
	int lastXor;
	ArrayList<Integer> usedHere;
	
	public void calculateHash(String s) {
		int h = 0;
		int xor = -100500;
		usedHere = new ArrayList<Integer>();
		for (int i = 0; i < s.length(); i++) {
			xor = h ^ s.charAt(i);
			usedHere.add(xor);
			h = T[xor];
		}
		hash = h;
		lastXor = xor;
	}
	
	public static int indexOf(int[] A, int x) {
		for (int i = 0; i < A.length; i++) {
			if (A[i] == x)
				return i;
		}
		return -1;
	}
	
	public boolean go() {
		mainloop: for (int i = 0; i < n; i++) {
			String s = kw[i].s;
			int ind = kw[i].i;
			calculateHash(s);

			if (hash == ind) {
				// Fuck yeah!
				continue mainloop;
			}
			boolean exch = swap(lastXor, indexOf(T, ind));
			if (exch) {
				calculateHash(s);
				if (hash != ind)
					return false;
				for (int xor : usedHere)
					used[xor] = true;
				continue mainloop;
			}
			//System.out.printf("%s (%d)\n", s, i);
			return false;
		}
		return true;
	}
	
	public void fillTable() {
		for (int i = 0; i < 256; i++) {
			T[i] = i;
		}
		Random r = new Random();
		int t, j;
		for (int i = 0; i < 256; i++) {
			j = r.nextInt(256);
			t = T[i];
			T[i] = T[j];
			T[j] = t;
		}
	}
	
	public void printTable(int[] T) {
		for (int i = -1; i < 16; i++) {
			out.printf("%6d", i);
		}
		out.println();
		for (int i = 0; i < 16; i++) {
			out.printf("%6d", i);
			for (int j = 0; j < 16; j++) {
				int ind = 16*i + j;
				int val = T[ind];
				if (val == -1)
					out.printf("%6s", "?");
				else {
					char c = (char)ind;
					boolean printable = Character.isJavaIdentifierPart(c);
					out.printf("%6s", String.format("%c:%d", printable ? c : ' ', val));
				}
			}
			out.println();
		}
	}

	public void solve() throws NumberFormatException, IOException {
		readTokens();
		T = new int[256];
		PearsonTimer timer = new PearsonTimer();
		timer.start();
		while(true) {
			fillTable();
			used = new boolean[256];
			Arrays.fill(used, false);
			boolean found = go();
			if (found)
				break;
		}
		timer.stop();
		System.out.println(timer);
		printTable(T);
		(new PearsonValidator(kw, T)).validate();
	}
}

class Token implements Comparable<Token> {
	String s;
	int i;
	
	public Token(String s, int i) {
		this.s = s;
		this.i = i;
	}
	
	public int compareTo(Token t2) {
		return this.s.length() - t2.s.length();
	}
	
	public String toString() {
		return String.format("%d:%s", this.i, this.s);
	}
}

class PearsonValidator {
	Token[] kw;
	int[] T;
	
	public PearsonValidator(Token[] kw, int[] T) {
		this.kw = kw;
		this.T = T;
	}
	
	public static boolean isPermutation(int[] T) {
		HashSet<Integer> hs = new HashSet<Integer>();
		for (int i = 0; i < 256; i++) {
			if (T[i] != -1) {
				if (hs.contains(T[i])) {
					System.out.println("Duplicate: " + T[i]);
					return false;
				}
				hs.add(T[i]);
			}
		}
		return true;
	}
	
	public static int pearsonHash(String s, int[] T) {
		int h = 0;
		for (int i = 0; i < s.length(); i++) {
			int xor = h ^ s.charAt(i);
			if (T[xor] == -1) {
				System.out.printf("Unbound cell: %d (%s)", xor, s);
				return -1;
			}
			h = T[xor];
		}
		return h;
	}
	
	public void validate() {
		if (!isPermutation(T)) {
			System.out.println("Not a permutation");
			return;
		}
		for (int i = 0; i < kw.length; i++) {
			String s = kw[i].s;
			int h = pearsonHash(s, T); 
			if (h != kw[i].i) {
				System.out.printf("FAILURE: i = %d, h = %d (%s)", i, h, s);
				return;
			}
		}
		System.out.println("SUCCESS");
	}
}

//haven't found anything in the standard library so reinventing the wheel

class PearsonTimer {
	public void start(){
		if ( fIsRunning ) {
			throw new IllegalStateException("Must stop before calling start again.");
		}
		//reset both start and stop
		fStart = System.currentTimeMillis();
		fStop = 0;
		fIsRunning = true;
		fHasBeenUsedOnce = true;
	}

	public void stop() {
		if ( !fIsRunning ) {
			throw new IllegalStateException("Cannot stop if not currently running.");
		}
		fStop = System.currentTimeMillis();
		fIsRunning = false;
	}

	public String toString() {
		validateIsReadable();
		return millisToShortDHMS(fStop - fStart);
	}

	public long toValue() {
		validateIsReadable();
		return fStop - fStart;
	}

	public static String millisToShortDHMS(long duration) {
		long days  = TimeUnit.MILLISECONDS.toDays(duration);
		duration -= TimeUnit.DAYS.toMillis(days);
		long hours = TimeUnit.MILLISECONDS.toHours(duration);
		duration -= TimeUnit.HOURS.toMillis(hours);
		long minutes = TimeUnit.MILLISECONDS.toMinutes(duration);
		duration -= TimeUnit.MINUTES.toMillis(minutes);
		long seconds = TimeUnit.MILLISECONDS.toSeconds(duration);
		duration -= TimeUnit.SECONDS.toMillis(seconds);
		long millis = duration;
		if (days == 0) {
			return String.format("%02d:%02d:%02d.%03d", hours, minutes, seconds, millis);
		}
		else {
			return String.format("%dd%02d:%02d:%02d.%03d", days, hours, minutes, seconds, millis);
		}
	}

	private long fStart;
	private long fStop;

	private boolean fIsRunning;
	private boolean fHasBeenUsedOnce;

	private void validateIsReadable() {
		if ( fIsRunning ) {
			String message = "Cannot read a stopwatch which is still running.";
			throw new IllegalStateException(message);
		}
		if ( !fHasBeenUsedOnce ) {
			String message = "Cannot read a stopwatch which has never been started.";
			throw new IllegalStateException(message);
		}
	}
}